'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _xml2js = require('xml2js');

var _xml2js2 = _interopRequireDefault(_xml2js);

var _PipParser = require('./PipParser');

var _PipParser2 = _interopRequireDefault(_PipParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var PackageXmlParser = function () {
    function PackageXmlParser() {
        _classCallCheck(this, PackageXmlParser);
    }

    _createClass(PackageXmlParser, [{
        key: 'parse',
        value: function parse(runner, done) {
            var _this = this;

            var that = this;
            this.pips = runner.options.pips;

            _async2.default.series([function (cb) {
                return _this.readXml(cb);
            }, function (cb) {
                return _this.parseInfo(cb);
            }, function (cb) {
                return _this.parsePips(cb);
            }, function (cb) {
                return _this.parseAdditionalPackages(cb);
            }], function () {
                runner.filesToPackage = that.filesToPackage;
                runner.xmlInfo = that.info;
                done();
            });
        }
    }, {
        key: 'readXml',
        value: function readXml(callback) {
            var that = this;

            _fs2.default.readFile('package.xml', function (err, data) {
                var parser = new _xml2js2.default.Parser();

                if (err) {
                    return callback(new Error("The package.xml could not be read."));
                }

                parser.parseString(data, function (err, result) {
                    if (err) {
                        return callback(new Error("The package.xml does not appear to be a valid XML document."));
                    }

                    that.xml = result;
                    callback();
                });
            });
        }
    }, {
        key: 'parseInfo',
        value: function parseInfo(callback) {
            var pack = this.xml.package;
            var info = {
                name: pack['$'].name,
                author: pack.authorinformation[0].author[0],
                version: pack.packageinformation[0].version[0]
            };

            this.info = info;

            callback();
        }
    }, {
        key: 'parsePips',
        value: function parsePips(callback) {
            var instructions = [];
            this.xml.package.instructions.forEach(function (element) {
                var instruction = element.instruction;
                instruction.forEach(function (element) {
                    instructions.push({
                        type: element.$.type,
                        path: element._
                    });
                });
            });

            var parser = new _PipParser2.default(this.pips);
            var fileList = parser.run(instructions);

            this.filesToPackage = fileList;

            callback();
        }
    }, {
        key: 'parseAdditionalPackages',
        value: function parseAdditionalPackages(callback) {
            var packages = [];

            var optionals = this.xml.package.optionalpackages;
            var requireds = this.xml.package.requiredpackages;

            var addPackagePaths = function addPackagePaths(paths) {
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = paths[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var path = _step.value;

                        var query = [];
                        if (path.requiredpackage) query = path.requiredpackage;
                        if (path.optionalpackage) query = path.optionalpackage;

                        var _iteratorNormalCompletion2 = true;
                        var _didIteratorError2 = false;
                        var _iteratorError2 = undefined;

                        try {
                            for (var _iterator2 = query[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                                var pack = _step2.value;

                                if (pack.$.file) packages.push(pack.$.file + '@');
                            }
                        } catch (err) {
                            _didIteratorError2 = true;
                            _iteratorError2 = err;
                        } finally {
                            try {
                                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                                    _iterator2.return();
                                }
                            } finally {
                                if (_didIteratorError2) {
                                    throw _iteratorError2;
                                }
                            }
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }
            };

            if (optionals) addPackagePaths(optionals);
            if (requireds) addPackagePaths(requireds);

            this.filesToPackage = this.filesToPackage.concat(packages);

            callback();
        }
    }]);

    return PackageXmlParser;
}();

exports.default = PackageXmlParser;