'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _TreeBuilder = require('./TreeBuilder');

var _fstream = require('fstream');

var _fstream2 = _interopRequireDefault(_fstream);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _tar = require('tar');

var _tar2 = _interopRequireDefault(_tar);

var _del = require('del');

var _del2 = _interopRequireDefault(_del);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Packager = function () {
    function Packager(files, packageInfo) {
        _classCallCheck(this, Packager);

        this.packageInfo = packageInfo;

        // Remove duplicates from array
        this.filesToPackage = files.filter(function (el, i, arr) {
            return arr.indexOf(el) === i;
        });
    }

    _createClass(Packager, [{
        key: 'run',
        value: function run(done, quiet) {
            var _this = this;

            _async2.default.series([function (cb) {
                return _this.findLocalFiles(cb);
            }, function (cb) {
                return _this.writeTreeStructure(quiet, cb);
            }, function (cb) {
                return _this.prepackage(cb);
            }, function (cb) {
                return _this.packageAll(cb);
            }, function (cb) {
                return _this.cleanup(cb);
            }, function (cb) {
                return _this.getFileStats(cb);
            }], function (err, results) {
                var filesize = results.pop();

                if (!quiet) {
                    console.log('-> ' + _chalk2.default.green.bold('Package generated') + ' (' + filesize + ')');
                }

                done(err, results);
            });
        }
    }, {
        key: 'getFileProcessingList',
        value: function getFileProcessingList() {
            return this.filesToPackage.map(function (item) {
                return {
                    original: item,
                    adjusted: item.replace(/\.tar$/i, '')
                };
            }).map(function (item) {
                return function (callback) {
                    if (!_glob2.default.hasMagic(item.adjusted)) {
                        item.paths = [item.adjusted];
                        callback(null, item);
                        return;
                    }

                    (0, _glob2.default)(item.adjusted, function (err, files) {
                        item.paths = files;
                        callback(err, item);
                    });
                };
            });
        }
    }, {
        key: 'getFileStats',
        value: function getFileStats(done) {
            _fs2.default.stat(this.packageInfo.name + '.tar', function (err, stats) {
                function bytesToSize(bytes) {
                    var sizes = ['Bytes', 'KB', 'MB', 'GB'];
                    if (bytes == 0) return '0 Byte';
                    var i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
                    return Math.round(bytes / Math.pow(1000, i), 2) + ' ' + sizes[i];
                };

                done(err, bytesToSize(stats.size));
            });
        }
    }, {
        key: 'findLocalFiles',
        value: function findLocalFiles(done) {
            var that = this;
            var fileProcessingList = this.getFileProcessingList();

            _async2.default.parallel(fileProcessingList, function (err, results) {
                var prepack = [];
                var direct = ['package.xml'];
                results.forEach(function (instruction) {
                    if (instruction.original.endsWith('.tar')) {
                        prepack = prepack.concat(instruction.paths);
                    } else {
                        direct = direct.concat(instruction.paths.map(function (i) {
                            return i.replace(/\.tar@$/, '.tar');
                        }));
                    }
                });

                that.packagingPlan = {
                    prepack: prepack, direct: direct
                };

                done();
            });
        }
    }, {
        key: 'prepackage',
        value: function prepackage(done) {
            var that = this;
            var tasks = [];

            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                var _loop = function _loop() {
                    var dir = _step.value;

                    tasks.push(function (cb) {
                        var packer = _tar2.default.Pack({ noProprietary: true, fromBase: true });
                        var writeStream = _fs2.default.createWriteStream(dir + '.tar');

                        packer.on('error', function (err) {
                            return cb(err);
                        }).on('end', function () {
                            return cb();
                        });
                        _fstream2.default.Reader({ path: dir, type: 'Directory' }).on('error', done).pipe(packer).pipe(writeStream);
                    });
                };

                for (var _iterator = this.packagingPlan.prepack[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    _loop();
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            _async2.default.parallel(tasks, function (err) {
                return done(err);
            });
        }
    }, {
        key: 'packageAll',
        value: function packageAll(done) {
            var that = this;
            var packer = _tar2.default.Pack({ noProprietary: true, fromBase: true });

            var streams = [];
            var files = this.packagingPlan.direct.concat(this.packagingPlan.prepack.map(function (item) {
                return item + '.tar';
            })).map(function (item) {
                return item.replace(_path2.default.sep, '/');
            }); // Windows compatibility

            var folders = [];
            files.forEach(function (dir) {
                var base = _path2.default.dirname(dir);
                var dirs = [base];
                while (base.includes('/') || base.includes('\\')) {
                    base = _path2.default.dirname(base);
                    dirs.push(base);
                }
                folders = folders.concat(dirs);
            });

            folders = folders.filter(function (el, i, arr) {
                return arr.indexOf(el) === i;
            });

            var readStream = _fstream2.default.Reader({
                path: process.cwd(),
                filter: function filter(entry) {
                    // Remove path up to cwd
                    var file = entry.path.replace(process.cwd() + _path2.default.sep, '').replace(/\\/g, '/');
                    return file == process.cwd().replace(/\\/g, '/') || folders.indexOf(file) !== -1 || files.indexOf(file) !== -1;
                }
            });

            readStream.pipe(packer).pipe(_fs2.default.createWriteStream(that.packageInfo.name + '.tar')).on('finish', function () {
                return done();
            });
        }
    }, {
        key: 'cleanup',
        value: function cleanup(done) {
            (0, _del2.default)(this.packagingPlan.prepack.map(function (i) {
                return i + '.tar';
            })).then(function (paths) {
                done();
            });
        }
    }, {
        key: 'writeTreeStructure',
        value: function writeTreeStructure(quiet, done) {
            var _tree$_;

            if (quiet) return done();

            var that = this;
            var tree = {};
            this.packagingPlan.direct.forEach(function (file) {
                tree = (0, _TreeBuilder.buildTree)(tree, file);
            });

            (_tree$_ = tree._).push.apply(_tree$_, _toConsumableArray(this.packagingPlan.prepack.map(function (i) {
                return i + '.tar';
            })));

            console.log(_chalk2.default.bold.green(this.packageInfo.name + '.tar'));
            (0, _TreeBuilder.outputTree)(tree);

            done();
        }
    }]);

    return Packager;
}();

exports.default = Packager;