'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.buildTree = buildTree;
exports.outputTree = outputTree;

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var log = console.log;

function buildTree(tree, item) {
    if (!tree._) tree._ = [];

    if (!item.includes('/')) tree._.push(item);else {
        var i = item.indexOf('/');
        var end = item.slice(i + 1);
        var folder = item.slice(0, i);

        if (!tree[folder]) tree[folder] = {};
        tree[folder] = buildTree(tree[folder], end);
    }

    return tree;
}

function outputTree(tree) {
    var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var levelsDone = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

    var files = tree._.concat(Object.keys(tree).filter(function (i) {
        return i != '_';
    })).sort();

    files.forEach(function (item, key) {
        var isFolder = tree.hasOwnProperty(item);
        var isLastEntry = key == files.length - 1;

        // The symbol to display.
        // This is different for items in the middle and at the end of a list
        var symbol = isLastEntry ? '└' : '├';

        // The prefix for this level.
        var prefix = '';
        for (var i = 0; i < level; i++) {
            prefix += levelsDone.indexOf(i) != -1 ? '    ' : '│   ';
        }

        // The item itself.
        var itemText = isFolder ? _chalk2.default.bold(item) : item.endsWith('.tar') ? _chalk2.default.bold.cyan(item) : item;

        // Log the item
        log(prefix + symbol + '── ' + itemText);

        // If the item was a folder, go over its files as well.
        if (isFolder) {
            if (isLastEntry) {
                levelsDone.push(level);
            }
            outputTree(tree[item], level + 1, levelsDone);
        }
    });
}